% Problem 4.3
%{
solve the Christoffel equation for bulk unidirectional composites: 
find eigenvalues, eigenvectors, and corresponding wavespeeds 
for a propagating direction rotated by an angle theta about the material z axis.
Numerical Example: CFRP_T300_914
%}
%% initialization
clc         % clear command window 
clear       % clear workspace
close all   % close all figures
format compact % compact format does not have empty lines between line of text
%%  initial data: material properties, units, conversion factors
deg=pi/180; km_s=1e-3; GPa=1e9;  % units and convertion factors
%% T300-914 unidirectional CFRP
[C,rho]=CFRP_T300_914;  % stiffness matrix, GPa; density, kg/m3 
display('unidirectional T300-914 CFRP results')
display(C/GPa, '(a) stiffness matrix, GPa');
%% convert from stiffness matrix to stiffness tensor
c=C2c(C);
% display(c);
%% define wavefront directions
% theta=30*deg; 
% theta=[10 30]*deg; 
theta=[0 10 15 30 45 60 75 90]*deg; 
Ntheta=size(theta,2);
vvv=zeros(Ntheta,3); VVV=zeros(3,3,Ntheta); DDD=zeros(1,Ntheta);
for ntheta=1:Ntheta
%% define wavefront unit vector n
n=[cos(theta(ntheta)) 0 sin(theta(ntheta))]; % unit vector normal to wave front
%% calculate Christoffel acoustic tensor Gamma
Gamma=zeros(3,3);
for i=1:3; 
    for m=1:3; sum=0; 
        for j=1:3; 
            for k=1:3; 
                sum=sum+c(i,j,k,m)*n(k)*n(j); 
            end; 
        end;
Gamma(i,m)=sum; 
    end; 
end;
% display(Gamma)
%% calculate eigenvalues and eigenvectors of Christoffel eqn
[U,D,V]=svd(Gamma); % gives eigenvalues in decreasing order
DD=diag(D); v(:)=sqrt(DD(:)/rho); % extract wavespeeds
%% normalize eigenvectors to make +ve the largest element in each column
Vabs=abs(V);[M,IV]=max(Vabs,[],1); ND=size(D,1); Vn=zeros(ND); 
for j=1:ND; scale=sign(V(IV(j),j)); Vn(:,j)=V(:,j)/scale; end
%% store wavespeeds and corresponding eigenvectors
vvv(ntheta,:)=v(:);
VVV(:,:,ntheta)=Vn(:,:);
%% display partial results
display('===============')
display(['theta = ' num2str(theta(ntheta)/deg) ' deg'])
% D=D
display(n,'wavefront direction')
display(Gamma/GPa, '(b) acoustic tensor, Gamma, GPa')
display(DD'/GPa,'(c) eigenvalues, GPa')
display(v*km_s,'(d) wavespeeds, v, km/s')
display(Vn,'(e) polarization vectors')
end
%% display results
% display(theta/deg,'theta, deg')
% display(vvv*km_s,'v, km/s')
% display(VVV,'polarization vectors')
% cS=sqrt(C(6,6)/rho)*km_s








