% Problem 4.7 LAMINA CHRISTOFFEL EQUATION FOR OFF-AXIS COMPOSITE LAMINA
%{
solve the Christoffel equation for an offaxis composite lamina 
of different angles theta:
for a given wavespeed, find alpha squared, eigenvalues, eigenvectors
repeat for other wavespeeds
a2=alpha^2; a=alpha;
v=wavespeed
%}
%% initialization
clc             % clear command window 
clear           % clear workspace
close all       % close all figures
format compact  % densifies command window output lines
nfig=1; % initialize figure count
%%  initial data: material properties, units, conversion factors, tolerances
deg=180/pi; km_s=1e-3;  % units and convertion factors
[C_,rho]=CFRP_T300_914;  % local stiffness matrix, GPa; density, kg/m3
tol_D=1e-6; % tolerance in D to find the eigenvectors U coresponding to zero D
%% rotate stiffness matrix through angle theta
% theta=0/deg;       % lamina angle
% theta=5/deg;       % lamina angle
% theta=10/deg;       % lamina angle
% theta=15/deg;       % lamina angle
% theta=20/deg;       % lamina angle
% theta=25/deg;       % lamina angle
% theta=30/deg;       % lamina angle
% theta=35/deg;       % lamina angle
% theta=40/deg;       % lamina angle
% theta=45/deg;       % lamina angle
% theta=50/deg;       % lamina angle
% theta=55/deg;       % lamina angle
% theta=60/deg;       % lamina angle
% theta=65/deg;       % lamina angle
% theta=70/deg;       % lamina angle
% theta=75/deg;       % lamina angle
% theta=80/deg;       % lamina angle
% theta=85/deg;       % lamina angle
% theta=90/deg;       % lamina angle
theta=[0 5 10 15 20 25 30 35 40 45 50 55 60 65 70 75 80 85 90]/deg;
                                                                % lamina angles
Nt=size(theta,2);
%% define wavespeed range
vm=10;       % minimum wavespeed, m/s (vmin<10 gives complex aa2 values 
                                                       % due to numerical error
vM=10e3;    % maximum wavespeed, m/s
% vm=1750;      % minimum wavespeed, m/s
% vM=1850;      % maximum wavespeed, m/s
% vm=2000;      % minimum wavespeed, m/s
% vM=5000;      % maximum wavespeed, m/s
% Nv=501;       % number of wavespeed points to be considered
Nv=51;          % number of wavespeed points to be considered
if Nv==1; v=vm; else dv=(vM)/(Nv-1); v=[vm dv:dv:vM]; end
%% or define wavespeed explicitly
% v=[0 1000 2000];
% v=[1900 1907 1920];
% v=[1500 2500 3500 4500];
% v=[5000 6000];
% v=[0 5000 8000 12000]; 
% v=[0 5000 8000 12000]; 
% v=[0 1000 3000 4000 6000 8000 12000]; 
% v=[0 1000 2000 4000 6000 8000 10000 12000]; 
% display(v(nv), 'wavespeed, km/s')
Nv=size(v,2);   
%% loop over all the angles theta
alpha2{Nt,1}=[];alpha{Nt,1}=[];Vectors{Nt,1}=[]; C_theta{Nt,1}=[];
                                % allocate space for the cell arrays
for nt=1:Nt
C=rot3D(C_,theta(nt));  % rotate the stiffness matrix
%% extract the stiffness coefficients needed in Christoffel equation
C11=C(1,1); C12=C(1,2); C13=C(1,3); C16=C(1,6);
C22=C(2,2); C23=C(2,3); C26=C(2,6);
C33=C(3,3); C36=C(3,6);
C44=C(4,4); C45=C(4,5); C55=C(5,5); C66=C(6,6);
%% loop over all wavespeeds to extract a2, eigenvalues, a, and eigenvectors, U 
% reserve space for eigenvalues
Na=6;Na2=Na/2; a=zeros(Na,Nv); aa2=zeros(Na2,Nv); 
U=zeros(3,Na,Nv); % reserve space for eigenvector
for nv=1:Nv % extract eigenvalues and eigenvectors at each wavespeed
%% write Christoffel eq in expanded form as a cubic eq in a2
rv2=rho*v(nv)^2; rv4=rv2^2; rv6=rv2^3; 
C11_rv2=C11-rv2; C55_rv2=C55-rv2; C66_rv2=C66-rv2;
A6=C33*C44*C55-C33*C45^2;
A4=(C55*C44-C45^2)*C55_rv2+C55*C33*C66_rv2+C44*C33*C11_rv2...
    -2*C16*C45*C33+2*(C45+C36)*(C13+C55)*C45-(C13+C55)^2*C44-(C45+C36)^2*C55;
A2=C33*C11_rv2*C66_rv2+C44*C11_rv2*C55_rv2+C55*C66_rv2*C55_rv2...
    -C11_rv2*(C45+C36)^2-C66_rv2*(C13+C55)^2-2*C55_rv2*C16*C45...
    +2*C16*(C45+C36)*(C13+C55)-C16^2*C33;
A0=(C11_rv2*C66_rv2-C16^2)*C55_rv2;
% display([[A6 A4 A2 A0]'],'A6 A4 A2 A0')
%% scale coefficients of Christoffel eq
sf=max([abs(A6) abs(A4) abs(A2) abs(A0)]); % scale factor for numerical work
% display(sf)
% sf=1; % trial value to see if sf has any influence on accuracy
A6n=A6/sf; A4n=A4/sf; A2n=A2/sf; A0n=A0/sf;
fChr=@(s) (A6*s.^3+A4*s.^2+A2*s+A0)/sf; % the cubic function 
%% solve Christoffel eq
a2=roots([A6n A4n A2n A0n]); f=fChr(a2); % polynomial roots
% display(v(nv),'v, m/s');     % display speed, m/s
% display([a2 f],'a2   f(a2)') % display roots and funct values at roots
%% store a2 as aa2
aa2(:,nv)=a2;  
%% calculate the 6 values of alpha
    a(1,nv)=sqrt(a2(1)); a(2,nv)=-sqrt(a2(1)); 
    a(3,nv)=sqrt(a2(2)); a(4,nv)=-sqrt(a2(2));  
    a(5,nv)=sqrt(a2(3)); a(6,nv)=-sqrt(a2(3));
%% Calculate eigenvectors
AChr=@(a)...
 [ C11_rv2+C55*a^2       C16+C45*a^2          (C13+C55)*a;
   C16+C45*a^2           C66_rv2+C44*a^2      (C36+C45)*a;
  (C13+C55)*a            (C36+C45)*a           C55_rv2+C33*a^2  ]; % function
    for na=1:Na % calculate eigenvectors at each alpha value
        A=AChr(a(na,nv)); An=A./max(max(abs(A)));
        % use eig function 
%         [V,Diag_eigenvalues]=eig(An);
        % use svd function
        [~,Diag_eigenvalues,V]=svd(An);
    %% find the zero eigenvalue
        indexD=0;
        for nD=1:3
            if (abs(Diag_eigenvalues(nD,nD))<=tol_D)
                indexD=nD;
            end
       end
    %% pick up in VV the eigenvector corresponding to the zero eigenvalue
    % store in U the normalized VV (max value = 1)  
    VV=V(:,indexD); [VVm,iVVm]=max(abs(VV));
%     U(:,na,nv)=VV;  % original normalization to norm=1
    U(:,na,nv)=VV/VV(iVVm);   % normalized to max=1
    end
end
%% pick up alpha2
alpha2{nt,1}=aa2;
alpha{nt,1}=a;
Vectors{nt,1}=U;
C_theta{nt,1}=C;
%% plot and display results
% figure(nfig); nfig=nfig+1;
% plot(v, alpha2(2,:),v, alpha2(3,:),v, alpha2(1,:));
% plot(v, aa2(2,:),'ob', v,aa2(3,:),'ob', v,aa2(1,:),'ob');
% xlabel('wavespeed, m/s'); ylabel('\alpha^{2}')% note greek and ^ tricks...
% title(['offaxis composite lamina \theta=',num2str(theta(nt)*deg),' deg'],...
%     'FontSize', 11, 'FontWeight','normal')
% ylim([-0.12 -0.10])
% ylim([-0.20 -0.05])
grid on
% display([nv,v(nv)],'nv, v(nv)')
% display(U(:,:,nv),'polarization eigenvectors stored in U')
end
for nt=1:Nt
    figure(nt); hold on; grid on
    for nD=1:3; scatter(v,alpha2{nt}(nD,:),'ob'); end
xlabel('wavespeed, m/s'); ylabel('\alpha^{2}')% note greek and ^ tricks...
title(['offaxis composite lamina \theta=',num2str(theta(nt)*deg),' deg'],...
    'FontSize', 11, 'FontWeight','normal')
end

%% verify results with Darun
% d_alpha2{Nt,1}=[];
% load('alpha2_Darun');
% for nt=1:Nt;
%     d_alpha2{nt,1}=alpha2{nt,1}-alpha2_Darun{nt,1};
% end
% save('d_alpha2')
% for nt=1:Nt
%     figure(nt+Nt); hold on; grid on
%     for nD=1:3; scatter(v,d_alpha2{nt}(nD,:),'ob'); end
% xlabel('wavespeed, m/s'); ylabel('d\alpha^{2}')% note greek and ^ tricks...
% title(['offaxis composite lamina \theta=',num2str(theta(nt)*deg),' deg'],...
%     'FontSize', 11, 'FontWeight','normal')
% end


























