% Problem 4.8 LAMINA CHRISTOFFEL EQUATION FOR FULLY ORTHOTROPIC COMPOSITE LAMINA
%{
solve the lamina Christoffel equation
for a given wavespeed, find alpha squared, eigenvalues, eigenvectors
repeat for other wavespeeds
a2=alpha^2; a=alpha;
v=wavespeed
%}
%% initialization
clc             % clear command window 
clear           % clear workspace
close all       % close all figures
format compact  % densifies command window output lines
nfig=1; % initialize figure count
%%  initial data: material properties, units, conversion factors, tolerances
deg=180/pi; km_s=1e-3;  % units and convertion factors
[C,rho]=CFRP_orthotropic;  % local stiffness matrix, GPa; density, kg/m3
tol_D=1e-6; % tolerance in D to find the eigenvectors U coresponding to zero D
%% define wavespeed range
vm=10;       % minimum wavespeed, m/s (vmin<10 gives complex aa2 values 
                                                       % due to numerical error
vM=10e3;    % maximum wavespeed, m/s
% vm=1750;      % minimum wavespeed, m/s
% vM=1850;      % maximum wavespeed, m/s
% vm=2000;      % minimum wavespeed, m/s
% vM=5000;      % maximum wavespeed, m/s
% Nv=501;       % number of wavespeed points to be considered
Nv=51;          % number of wavespeed points to be considered
if Nv==1; v=vm; else dv=(vM)/(Nv-1); v=[vm dv:dv:vM]; end
%% or define wavespeed explicitly
% v=[0 1000 2000];
% v=[1900 1907 1920];
% v=[1500 2500 3500 4500];
% v=[5000 6000];
% v=[0 5000 8000 12000]; 
% v=[0 5000 8000 12000]; 
% v=[0 1000 3000 4000 6000 8000 12000]; 
% v=[0 1000 2000 4000 6000 8000 10000 12000]; 
% display(v(nv), 'wavespeed, km/s')
Nv=size(v,2);   
%% reserve space for eigenvalues and eigenvectors
Na=6;Na2=Na/2; 
a=zeros(Na,Nv); aa2=zeros(Na2,Nv); UU=zeros(Na2,Na,Nv); 
%% extract the stiffness coefficients needed in Christoffel equation
C11=C(1,1); C12=C(1,2); C13=C(1,3); C16=C(1,6);
C22=C(2,2); C23=C(2,3); C26=C(2,6);
C33=C(3,3); C36=C(3,6);
C44=C(4,4); C45=C(4,5); C55=C(5,5); C66=C(6,6);
%% loop over all wavespeeds to extract a2, eigenvalues, a, and eigenvectors, U 
for nv=1:Nv % extract eigenvalues and eigenvectors at each wavespeed
%% write Christoffel eq in expanded form as a cubic eq in a2
rv2=rho*v(nv)^2; rv4=rv2^2; rv6=rv2^3; 
C11_rv2=C11-rv2; C55_rv2=C55-rv2; C66_rv2=C66-rv2;
A6=C33*C44*C55-C33*C45^2;
A4=(C55*C44-C45^2)*C55_rv2+C55*C33*C66_rv2+C44*C33*C11_rv2...
    -2*C16*C45*C33+2*(C45+C36)*(C13+C55)*C45-(C13+C55)^2*C44-(C45+C36)^2*C55;
A2=C33*C11_rv2*C66_rv2+C44*C11_rv2*C55_rv2+C55*C66_rv2*C55_rv2...
    -C11_rv2*(C45+C36)^2-C66_rv2*(C13+C55)^2-2*C55_rv2*C16*C45...
    +2*C16*(C45+C36)*(C13+C55)-C16^2*C33;
A0=(C11_rv2*C66_rv2-C16^2)*C55_rv2;
% display([[A6 A4 A2 A0]'],'A6 A4 A2 A0')
%% scale coefficients of Christoffel eq
sf=max([abs(A6) abs(A4) abs(A2) abs(A0)]); % scale factor for numerical work
% display(sf)
% sf=1; % trial value to see if sf has any influence on accuracy
A6n=A6/sf; A4n=A4/sf; A2n=A2/sf; A0n=A0/sf;
fChr=@(s) (A6*s.^3+A4*s.^2+A2*s+A0)/sf; % the cubic function 
%% solve Christoffel eq
a2=roots([A6n A4n A2n A0n]); f=fChr(a2); % polynomial roots
a2=sort(a2,'descend');   % sort roots in descending order
% display(v(nv),'v, m/s');     % display speed, m/s
% display([a2 f],'a2   f(a2)') % display roots and funct values at roots
%% store a2 as aa2
aa2(:,nv)=a2;  
%% calculate the 6 values of alpha
    a(1,nv)=sqrt(a2(1)); a(2,nv)=-sqrt(a2(1)); 
    a(3,nv)=sqrt(a2(2)); a(4,nv)=-sqrt(a2(2));  
    a(5,nv)=sqrt(a2(3)); a(6,nv)=-sqrt(a2(3));
%% Calculate eigenvectors
AChr=@(a)...
 [ C11_rv2+C55*a^2       C16+C45*a^2          (C13+C55)*a;
   C16+C45*a^2           C66_rv2+C44*a^2      (C36+C45)*a;
  (C13+C55)*a            (C36+C45)*a           C55_rv2+C33*a^2  ]; % function
    for na=1:Na % calculate eigenvectors at each alpha value
        A=AChr(a(na,nv)); An=A./max(max(abs(A)));
        % use eig function 
%         [V,Diag_eigenvalues]=eig(An);
        % use svd function
        [~,Diag_eigenvalues,V]=svd(An);
    %% find the zero eigenvalue
        indexD=0;
        for nD=1:3
            if (abs(Diag_eigenvalues(nD,nD))<=tol_D)
                indexD=nD;
            end
       end
    %% pick up in VV the eigenvector corresponding to the zero eigenvalue
    % store in UU the normalized VV (max value = 1)  
    VV=V(:,indexD); [VVm,iVVm]=max(abs(VV));
%     U(:,na,nv)=VV;  % original normalization to norm=1
    UU(:,na,nv)=VV/VV(iVVm);   % normalized to max=1
    end
end
%% plot and display results
% display([nv,v(nv)],'nv, v(nv)')
% display(UU(:,:,nv),'polarization eigenvectors stored in U')
% end
figure(nfig); nfig=nfig+1; hold on; grid on
for nD=1:Na2; scatter(v,aa2(nD,:),'ob'); end
xlabel('wavespeed, m/s'); ylabel('\alpha^{2}')% note greek and ^ tricks...
title('CFRP fully orthotropic composite lamina',...
    'FontSize', 11, 'FontWeight','normal')
%% organize the a2 values for plotting
%{ 
sort the a2 values at each v according to the resemblance between 
current eigenvector and one of the eigenvectors of previous v;
use only the odd-number eigenvectors (we do NOT sort the a's, we sort the a2's!)
%}
I_select=zeros(Na2,Nv); alpha2=zeros(Na2,Nv); U=zeros(Na2,Na,Nv); % reserve space
I_select(:,1)=[1 2 3]; 
% first column is the natural order
alpha2(:,1)=aa2(:,1);
U(:,:,1)=UU(:,:,1);
% the P single root remain the same
alpha2(3,:)=aa2(3,:); 
U(:,5,:)=UU(:,5,:);
U(:,6,:)=UU(:,6,:);
for nv=2:Nv % loop over all wavespeed pairs
% reserve space
dot_mag=zeros(Na2,1); 
Na2r=2; % reduce search only to the quasi SH&SV roots
    for na2=1:Na2r % loop over the SH,SV eigenvectors
        na_odd=2*na2-1; % the odd indices 1,3,5
        for na1=1:Na2 % compare each eigenvector with all previous eigenvectors
            na1_odd=2*na1-1; % the odd indices 1,3,5
            dot_mag(na1)=abs(dot(UU(:,na_odd,nv),UU(:,na1_odd,nv-1)));
        end
        [~,I_mag]=max(dot_mag);
       I_select(na2,nv)=I_select(I_mag,nv-1);
%% pick up alpha2 and Ufrom a2 and UU according to I_select indices
alpha2(I_select(na2,nv),nv)=aa2(na2,nv); 
U(:,2*I_select(na2,nv)-1,nv)=UU(:,2*na2-1,nv);
U(:,2*I_select(na2,nv),nv)=UU(:,2*na2,nv);
    end    
end
%% plot and display results
figure(nfig); nfig=nfig+1;
plot(v, alpha2(1,:),v, alpha2(2,:),v, alpha2(3,:)); grid on
xlabel('wavespeed, m/s'); ylabel('\alpha^{2}')% note greek and ^ tricks...
legend('\alpha^2_{12}','\alpha^2_{34}','\alpha^2_{56}')
title('CFRP fully orthotropic composite lamina',...
                        'FontSize', 11, 'FontWeight','normal')























